/* Bones borrowed from Li Zheng's 2014 China Japan visualization
*/

import peasy.*;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;

int startYear = 2017;
int endYear = 2023;
int startMonth = 0;
int endMonth = 11;
int numRows;
boolean showBox = true;
boolean showLabels = true;

int hHue = 16;
int hSaturation = 240;
int hBright = 220;
int hTrans = 160;

int hsHue = 163;
int hsSaturation = 240;
int hsBright = 220;
int hsTrans = 160;

int lHue = 210;
int lSaturation = 240;
int lBright = 220;
int lTrans = 160;

int tHue = 60;
int tSaturation = 240;
int tBright = 220;
int tTrans = 160;

int oHue = 120;
int oSaturation = 240;
int oBright = 220;
int oTrans = 160;

int iHue = 0;
int iSaturation = 0;
int iBright = 10;
int iTrans = 40;

color hColor = color(hHue, hSaturation, hBright, hTrans);
color hsColor = color(hsHue, hsSaturation, hsBright, hsTrans);
color tColor = color(tHue, tSaturation, tBright, tTrans);
color lColor = color(lHue, lSaturation, lBright, lTrans); 
color oColor = color(oHue, oSaturation, oBright, oTrans);
color iColor = color(iHue, iSaturation, iBright, iTrans);


int totalYear = endYear-startYear+1;

int hCount[][] = new int[totalYear][endMonth+1];
int hsCount[][] =  new int[totalYear][endMonth+1];
int oCount[][] =  new int[totalYear][endMonth+1];
int lCount[][] = new int[totalYear][endMonth+1];
int tCount[][] =  new int[totalYear][endMonth+1];

int hInv[][] = new int[totalYear][endMonth+1];
int hsInv[][] =  new int[totalYear][endMonth+1];
int oInv[][] =  new int[totalYear][endMonth+1];
int lInv[][] = new int[totalYear][endMonth+1];
int tInv[][] =  new int[totalYear][endMonth+1];

int hCountActual[][] = new int[totalYear][endMonth+1];
int hsCountActual[][] =  new int[totalYear][endMonth+1];
int oCountActual[][] =  new int[totalYear][endMonth+1];
int lCountActual[][] = new int[totalYear][endMonth+1];
int tCountActual[][] =  new int[totalYear][endMonth+1];

int hInvActual[][] = new int[totalYear][endMonth+1];
int hsInvActual[][] =  new int[totalYear][endMonth+1];
int oInvActual[][] =  new int[totalYear][endMonth+1];
int lInvActual[][] = new int[totalYear][endMonth+1];
int tInvActual[][] =  new int[totalYear][endMonth+1];

float hZs[][][] = new float[totalYear][endMonth+1][4];
float hsZs[][][] =  new float[totalYear][endMonth+1][4];
float oZs[][][] =  new float[totalYear][endMonth+1][4];
float lZs[][][] = new float[totalYear][endMonth+1][4];
float tZs[][][] =  new float[totalYear][endMonth+1][4];

float hInvZs[][][] = new float[totalYear][endMonth+1][4];
float hsInvZs[][][] =  new float[totalYear][endMonth+1][4];
float oInvZs[][][] =  new float[totalYear][endMonth+1][4];
float lInvZs[][][] = new float[totalYear][endMonth+1][4];
float tInvZs[][][] =  new float[totalYear][endMonth+1][4];

Table table;
//list of Items which will store each row's data as our Item class
ArrayList<Item> items = new ArrayList<Item>();

int X = 1600; //be the same as the size(x);
int Y = 1200; //be the same as the size(y);
int Z = 9000; //peasycam Z depth

int hHeight = 25; 
int hsHeight = 25; 
int oHeight = 25; 
int lHeight = 25; 
int tHeight = 25; 
  
float zBase = 20; 
float zDisplace = 0;
float adjustX = float(X)/(endMonth-startMonth);
float adjustY = float((Y-10))/(totalYear-1);
String[] monthNames = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

void setup() {
  size(1600, 1200, P3D);
  cam = new PeasyCam(this, X/2, Y/2, 600, 9000);
  cam.setMinimumDistance(1);//1
  cam.setMaximumDistance(Z);
  cam.rotateY(-0.2);
  cam.rotateX(-1.3);
  cam.rotateZ(0.2);
  
  float fov = PI/8;
  float cameraZ = (height/2.0) / tan(PI/6);
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*20.0); 

  cp5 = new ControlP5(this);
  setGui();
}
  
void initializeData() {
  
  table = loadTable("monthly.csv", "header");
  numRows = table.getRowCount();
  
  int year = 0;
  int mt = 0;
  //process headphones datad
  for (int i = 0; i < numRows/5; i++) {    
    String materialType = table.getString(i,0);
    int couts = table.getInt(i,2);
    int invTotal = table.getInt(i,3);
    if (couts==0){
      hCount[year][mt] = 0;//
      
    }else {hCount[year][mt] = hHeight*int(log(couts));} //}
    
    if(invTotal== 0) {
      hInv[year][mt] = 0;}
    else{hInv[year][mt] = hHeight * int(log(invTotal));}
    hCountActual[year][mt] = couts;
    hInvActual[year][mt] = invTotal;
    if(i% 12 == 11){
      mt = 0;
    year ++;}
    else{
      mt ++;}
    }
    
  year = 0;
  mt = 0;
  //process hs data
  for (int i = numRows/5; i < numRows/5*2; i++) {
    String materialType = table.getString(i,0);
    int couts = table.getInt(i,2);
    int invTotal = table.getInt(i,3);
    if (couts==0){
      hsCount[year][mt] = 0;//
      
    }else {hsCount[year][mt] = hsHeight*int(log(couts));} //}
    
    if(invTotal== 0) {
      hsInv[year][mt] = 0;}
    else{hsInv[year][mt] = hsHeight * int(log(invTotal));}
    
    hsCountActual[year][mt] = couts;
    hsInvActual[year][mt] = invTotal;
    if(i% 12 == 11){
      mt =0;
    year ++;}
    else{
      mt ++;}
  }
  
  year = 0;
  mt = 0;
  for (int i = numRows/5*2; i < numRows/5*3; i++) {
    String materialType = table.getString(i,0);
    int couts = table.getInt(i,2);
    int invTotal = table.getInt(i,3);
    if (couts==0){
      lCount[year][mt] = 0;//
      
    }else {lCount[year][mt] = lHeight*int(log(couts));} //}
    
    if(invTotal== 0) {
      lInv[year][mt] = 0;}
    else{lInv[year][mt] = lHeight * int(log(invTotal));}
    lCountActual[year][mt] = couts;
    lInvActual[year][mt] = invTotal;
    if(i% 12 == 11){
      mt =0;
    year ++;}
    else{
      mt ++;}
  }
  
  year = 0;
  mt = 0;
  for (int i = numRows/5*3; i < numRows/5*4; i++) {
    String materialType = table.getString(i,0);
    int couts = table.getInt(i,2);
    int invTotal = table.getInt(i,3);
    if (couts==0){
      oCount[year][mt] = 0;//
      
    }else {oCount[year][mt] = oHeight*int(log(couts));} //}
    
    if(invTotal== 0) {
      oInv[year][mt] = 0;}
    else{oInv[year][mt] = oHeight * int(log(invTotal));}
    oCountActual[year][mt] = couts;
    oInvActual[year][mt] = invTotal;
    if(i% 12 == 11){
      mt =0;
    year ++;}
    else{
      mt ++;}
      
  }
  
  year = 0;
  mt = 0;
  for (int i = numRows/5*4; i < numRows; i++) {
    String materialType = table.getString(i,0);
    int couts = table.getInt(i,2);
    int invTotal = table.getInt(i,3);
    if (couts==0){
      tCount[year][mt] = 0;//
      
    }else {tCount[year][mt] = tHeight*int(log(couts));} //}
    
    if(invTotal == 0) {
      tInv[year][mt] = 0;}
    else{tInv[year][mt] = tHeight * int(log(invTotal));}
    tCountActual[year][mt] = couts;
    tInvActual[year][mt] = invTotal;
    if(i% 12 == 11){
      mt =0;
    year ++;}
    else{
      mt ++;}
  }
}

void drawLaptops() {
  float z1, z2, z3, z4, x1, x2, y1, y2;
  lColor = color(lHue, lSaturation, lBright, lTrans);
  fill(lColor);
  zDisplace = 1400;
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      beginShape(QUADS);
      z1 = min(lCount[j][i], Z)+zBase+zDisplace;
      z2 = min(lCount[j+1][i], Z)+zBase+zDisplace;
      z3 = min(lCount[j+1][i+1], Z)+zBase+zDisplace;
      z4 = min(lCount[j][i+1], Z)+zBase+zDisplace;

      x1 = (i-startMonth)*adjustX;
      x2 = (i-startMonth+1)*adjustX;
      y1 = j*adjustY;
      y2 = (j+1)*adjustY;
      vertex(x1, y1, z1);
      vertex(x1, y2, z2);
      vertex(x2, y2, z3);
      vertex(x2, y1, z4);
      endShape();
      lZs[j][i][0] = z1;
      lZs[j][i][1] = z2;
      lZs[j][i][2] = z3;
      lZs[j][i][3] = z4;
    }
  }
}

void drawHeadphones() {
  
  float z1, z2, z3, z4, x1, x2, y1, y2;

  hColor = color(hHue, hSaturation, hBright, hTrans);
  fill(hColor);
  zDisplace = 1050;
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      beginShape(QUADS);
      z1 = min(hCount[j][i], Z)+zBase+zDisplace;
      z2 = min(hCount[j+1][i], Z)+zBase+zDisplace;
      z3 = min(hCount[j+1][i+1], Z)+zBase+zDisplace;
      z4 = min(hCount[j][i+1], Z)+zBase+zDisplace;

      x1 = (i-startMonth)*adjustX;
      x2 = (i-startMonth+1)*adjustX;
      y1 = j*adjustY;
      y2 = (j+1)*adjustY;
      vertex(x1, y1, z1);
      vertex(x1, y2, z2);
      vertex(x2, y2, z3);
      vertex(x2, y1, z4);
      endShape();
      hZs[j][i][0] = z1;
      hZs[j][i][1] = z2;
      hZs[j][i][2] = z3;
      hZs[j][i][3] = z4;
    }
  }
}

void drawHotSpots() {
  
  float z1, z2, z3, z4, x1, x2, y1, y2;
  zDisplace = 700;
  hsColor = color(hsHue, hsSaturation, hsBright, hsTrans);
  fill(hsColor);
  
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      beginShape(QUADS);
      z1 = min(hsCount[j][i], Z)+zBase+zDisplace;
      z2 = min(hsCount[j+1][i], Z)+zBase+zDisplace;
      z3 = min(hsCount[j+1][i+1], Z)+zBase+zDisplace;
      z4 = min(hsCount[j][i+1], Z)+zBase+zDisplace;

      x1 = (i-startMonth)*adjustX;
      x2 = (i-startMonth+1)*adjustX;
      y1 = j*adjustY;
      y2 = (j+1)*adjustY;
      vertex(x1, y1, z1);
      vertex(x1, y2, z2);
      vertex(x2, y2, z3);
      vertex(x2, y1, z4);
      endShape();
      hsZs[j][i][0] = z1;
      hsZs[j][i][1] = z2;
      hsZs[j][i][2] = z3;
      hsZs[j][i][3] = z4;
    }
  }
}

void drawTablets() {
  
  float z1, z2, z3, z4, x1, x2, y1, y2;
  zDisplace = 350;
  tColor = color(tHue, tSaturation, tBright, tTrans);
  fill(tColor);
  
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      beginShape(QUADS);
      z1 = min(tCount[j][i], Z)+zBase+zDisplace;
      z2 = min(tCount[j+1][i], Z)+zBase+zDisplace;
      z3 = min(tCount[j+1][i+1], Z)+zBase+zDisplace;
      z4 = min(tCount[j][i+1], Z)+zBase+zDisplace;

      x1 = (i-startMonth)*adjustX;
      x2 = (i-startMonth+1)*adjustX;
      y1 = j*adjustY;
      y2 = (j+1)*adjustY;
      vertex(x1, y1, z1);
      vertex(x1, y2, z2);
      vertex(x2, y2, z3);
      vertex(x2, y1, z4);
      endShape();
      tZs[j][i][0] = z1;
      tZs[j][i][1] = z2;
      tZs[j][i][2] = z3;
      tZs[j][i][3] = z4;
    }
  }
}

void drawOther() {
  zDisplace = 0;
  float z1, z2, z3, z4, x1, x2, y1, y2;

  oColor = color(oHue, oSaturation, oBright, oTrans);
  fill(oColor);
  
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      beginShape(QUADS);
      z1 = min(oCount[j][i], Z)+zBase+zDisplace;
      z2 = min(oCount[j+1][i], Z)+zBase+zDisplace;
      z3 = min(oCount[j+1][i+1], Z)+zBase+zDisplace;
      z4 = min(oCount[j][i+1], Z)+zBase+zDisplace;
     
      x1 = (i-startMonth)*adjustX;
      x2 = (i-startMonth+1)*adjustX;
      y1 = j*adjustY;
      y2 = (j+1)*adjustY;
      vertex(x1, y1, z1);
      vertex(x1, y2, z2);
      vertex(x2, y2, z3);
      vertex(x2, y1, z4);
      endShape();
      oZs[j][i][0] = z1;
      oZs[j][i][1] = z2;
      oZs[j][i][2] = z3;
      oZs[j][i][3] = z4;
    }
  }
}

void drawInv() {
  int k = 0;
  zDisplace = 350;
  fill(iColor);
  stroke(0,0,20,40);
  float z1, z2, z3, z4, x1, x2, y1, y2;
  for (int i = 0; i < endMonth; i++) {
    for (int j = 0; j < totalYear - 1; j++) {
      k = 0;
      beginShape(QUADS);
          z1 = min(oInv[j][i], Z)+zBase+zDisplace*k;
          z2 = min(oInv[j+1][i], Z)+zBase+zDisplace*k;
          z3 = min(oInv[j+1][i+1], Z)+zBase+zDisplace*k;
          z4 = min(oInv[j][i+1], Z)+zBase+zDisplace*k;
    
          x1 = (i-startMonth)*adjustX;
          x2 = (i-startMonth+1)*adjustX;
          y1 = j*adjustY;
          y2 = (j+1)*adjustY;
          vertex(x1, y1, z1);
          vertex(x1, y2, z2);
          vertex(x2, y2, z3);
          vertex(x2, y1, z4);
          endShape();
          oInvZs[j][i][0] = z1;
          oInvZs[j][i][1] = z1;
          oInvZs[j][i][2] = z1;
          oInvZs[j][i][3] = z1;
       k ++;
      beginShape(QUADS);
          z1 = min(tInv[j][i], Z)+zBase+zDisplace*k;
          z2 = min(tInv[j+1][i], Z)+zBase+zDisplace*k;
          z3 = min(tInv[j+1][i+1], Z)+zBase+zDisplace*k;
          z4 = min(tInv[j][i+1], Z)+zBase+zDisplace*k;
    
          x1 = (i-startMonth)*adjustX;
          x2 = (i-startMonth+1)*adjustX;
          y1 = j*adjustY;
          y2 = (j+1)*adjustY;
          vertex(x1, y1, z1);
          vertex(x1, y2, z2);
          vertex(x2, y2, z3);
          vertex(x2, y1, z4);
          endShape();
          tInvZs[j][i][0] = z1;
          tInvZs[j][i][1] = z2;
          tInvZs[j][i][2] = z3;
          tInvZs[j][i][3] = z4;
      k++;
      beginShape(QUADS);
          z1 = min(hsInv[j][i], Z)+zBase+zDisplace*k;
          z2 = min(hsInv[j+1][i], Z)+zBase+zDisplace*k;
          z3 = min(hsInv[j+1][i+1], Z)+zBase+zDisplace*k;
          z4 = min(hsInv[j][i+1], Z)+zBase+zDisplace*k;
    
          x1 = (i-startMonth)*adjustX;
          x2 = (i-startMonth+1)*adjustX;
          y1 = j*adjustY;
          y2 = (j+1)*adjustY;
          vertex(x1, y1, z1);
          vertex(x1, y2, z2);
          vertex(x2, y2, z3);
          vertex(x2, y1, z4);
          endShape();
          hsInvZs[j][i][0] = z1;
          hsInvZs[j][i][1] = z2;
          hsInvZs[j][i][2] = z3;
          hsInvZs[j][i][3] = z4;
       k ++;
      beginShape(QUADS);
          z1 = min(hInv[j][i], Z)+zBase+zDisplace*k;
          z2 = min(hInv[j+1][i], Z)+zBase+zDisplace*k;
          z3 = min(hInv[j+1][i+1], Z)+zBase+zDisplace*k;
          z4 = min(hInv[j][i+1], Z)+zBase+zDisplace*k;
    
          x1 = (i-startMonth)*adjustX;
          x2 = (i-startMonth+1)*adjustX;
          y1 = j*adjustY;
          y2 = (j+1)*adjustY;
          vertex(x1, y1, z1);
          vertex(x1, y2, z2);
          vertex(x2, y2, z3);
          vertex(x2, y1, z4);
          endShape();
          hInvZs[j][i][0] = z1;
          hInvZs[j][i][1] = z2;
          hInvZs[j][i][2] = z3;
          hInvZs[j][i][3] = z4;
      k ++;
      beginShape(QUADS);
          z1 = min(lInv[j][i], Z)+zBase+zDisplace*k;
          z2 = min(lInv[j+1][i], Z)+zBase+zDisplace*k;
          z3 = min(lInv[j+1][i+1], Z)+zBase+zDisplace*k;
          z4 = min(lInv[j][i+1], Z)+zBase+zDisplace*k;
    
          x1 = (i-startMonth)*adjustX;
          x2 = (i-startMonth+1)*adjustX;
          y1 = j*adjustY;
          y2 = (j+1)*adjustY;
          vertex(x1, y1, z1);
          vertex(x1, y2, z2);
          vertex(x2, y2, z3);
          vertex(x2, y1, z4);
          endShape();
          lInvZs[j][i][0] = z1;
          lInvZs[j][i][1] = z2;
          lInvZs[j][i][2] = z3;
          lInvZs[j][i][3] = z4;
    }
  }
}

void draw() {
  background(255);
  colorMode(HSB, 255);
  initializeData();

  drawHeadphones();
  drawHotSpots();
  drawLaptops();
  drawTablets();
  drawOther();
  drawInv();
  drawMouseLabels();
  pushMatrix();
  textSize(30);
  drawLabels();
  popMatrix();
  gui();
}

void drawLabels() {
 //maybe declare better font here
 zDisplace = 350;
 //rotate(PI/2);
 int offset = int(zDisplace/3);
 
    int ticks = 10;
    for (int i = 0; i <= totalYear - 1; i++) {
    fill(0);
    pushMatrix();
    translate(-50, i*adjustY+10, 0);
    rotateX(-PI/2);
    text(i+2017, 0 , 0, 0);
    popMatrix();   
    
    }
    //draw X
    float adjustX = float(X)/(endMonth-startMonth);
    for (int i = startMonth; i <= endMonth; i++) {
      fill(0);
      pushMatrix();
      translate((i-startMonth)*adjustX, Y+2*ticks, 0);
      rotateX(-PI/2);
      textAlign(CENTER);
      textSize(30);
      text(monthNames[i], 0, 0);
      popMatrix();
  }
  
  textSize(60);
  String[] labels = {"Laptops", "Headphones", "Hot Spots", "Tablets", "Other"}; // thanks jenni!
  for (int i=4;i>=0; i--) {
    pushMatrix();
    translate(-200, Y/2, zDisplace*i+offset);
    rotateX(-PI/2);
    text(labels[4-i], 0,0,0);
    popMatrix(); 
  }
}


void drawMouseLabels() {
  int zDisplace = 350;
  int zBase = 20;
  table = loadTable("monthly.csv", "header");
  numRows = table.getRowCount();
  
  int [][][] coutValues = {tCountActual, oCountActual, hsCountActual, hCountActual, lCountActual};
  int [][][] invValues =  {tInvActual, oInvActual, hsInvActual, hInvActual, lInvActual};
  
  int [][][] coutHeights = {tCount, oCount, hsCount, hCount, lCount};
  int [][][] invHeights =  {tInv, oInv, hsInv, hInv, tInv};
  
  float [][][][] coutZs = {oZs, tZs, hsZs, hZs, lZs};
  float [][][][] invZs = {oInvZs, tInvZs, hsInvZs, hInvZs, lInvZs};
  
  for (int i=0; i<totalYear; i++){ 
    for (int j=0; j<endMonth; j++) {
      for (int k=0; k<5; k++) {
        //int countZ = coutHeights[k][i][j] + zBase*k + zDisplace * k;
        //int invZ = invHeights[k][i][j] + zBase*k + zDisplace * k;
        int coutValue = coutValues[k][i][j];
        int invValue = invValues[k][i][j];
        float [] zcPos = coutZs[k][i][j];
        float [] ziPos = invZs[k][i][j];
      
        //float y1 = i*adjustY;
        //float y2 = (i+1)*adjustY;
        //float x1 = j*adjustX;
        //float x2 = (j+1)*adjustX;
        float z = max(zcPos[0], ziPos[0]);
        //float z1 = min(zcPos);
        //float z2 = max(zcPos);
        float y = i*adjustY;
        float x = j*adjustX;
        
        //if (mouseX>screenX(x1,y1,z1) & mouseX<screenX(x2,y1,z1) & mouseY>screenY(x1,y1,z1) & mouseY<screenY(x1,y2,z1)){
        //  fill(255);
        //  textSize(14);
        //  String label = "Checkouts: " + str(coutValue) + "\n" + "Inventory: " + str(invValue);
        // max and min relative screen X values for all 4 of these points and then say is mouseX within this? and same with Y
      
        float mouseObjectDistance = sq(mouseX-screenX(x,y,z)) + sq(mouseY-screenY(x,y,z));
        
        if (mouseObjectDistance < 35) {
         fill(0);
         textSize(14);
         String label = "Checkouts: " + str(coutValue) + "\n" + "Inventory: " + str(invValue);
         //beginShape(QUADS);
          
         // vertex(mouseX, mouseY,z1);
         // vertex(mouseX, mouseY, z2);
         // vertex(mouseX+20, mouseY+20, z1);
         // vertex(mouseX+20, mouseY+20, z2);
         // endShape();
        hint(DISABLE_DEPTH_TEST);
        cam.beginHUD();
        text(label, mouseX, mouseY);
        cam.endHUD();
        hint(ENABLE_DEPTH_TEST);
        }
      }
    }
  }
}

void setGui(){
  PFont pfont= createFont("Arial", 16, true);
  ControlFont font = new ControlFont(pfont, 12);
  
  cp5.addTextlabel("title")
  .setText("Same-Day Checkouts of Equipment at the SPL")
  .setPosition(X/3-60, 20)
  .setFont(new ControlFont(pfont, 30))
  .setColorValue(color(0,0,0,255));
  
  cp5.addTextlabel("subtitle")
  .setText("2017-2023")
  .setPosition(X/2-100, 70)
  .setFont(new ControlFont(pfont, 20))
  .setColorValue(color(0,0,0,255));
  
  cp5.setAutoDraw(false);
  
}

void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}
